FFmpeg
Loading...
Searching...
No Matches
resample.c
Go to the documentation of this file.
1/*
2 * audio resampling
3 * Copyright (c) 2004-2012 Michael Niedermayer <michaelni@gmx.at>
4 * bessel function: Copyright (c) 2006 Xiaogang Zhang
5 *
6 * This file is part of FFmpeg.
7 *
8 * FFmpeg is free software; you can redistribute it and/or
9 * modify it under the terms of the GNU Lesser General Public
10 * License as published by the Free Software Foundation; either
11 * version 2.1 of the License, or (at your option) any later version.
12 *
13 * FFmpeg is distributed in the hope that it will be useful,
14 * but WITHOUT ANY WARRANTY; without even the implied warranty of
15 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
16 * Lesser General Public License for more details.
17 *
18 * You should have received a copy of the GNU Lesser General Public
19 * License along with FFmpeg; if not, write to the Free Software
20 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
21 */
22
23/**
24 * @file
25 * audio resampling
26 * @author Michael Niedermayer <michaelni@gmx.at>
27 */
28
29#include "libavutil/avassert.h"
30#include "libavutil/mem.h"
31#include "resample.h"
32
33/**
34 * builds a polyphase filterbank.
35 * @param factor resampling factor
36 * @param scale wanted sum of coefficients for each filter
37 * @param filter_type filter type
38 * @param kaiser_beta kaiser window beta
39 * @return 0 on success, negative on error
40 */
41static int build_filter(ResampleContext *c, void *filter, double factor, int tap_count, int alloc, int phase_count, int scale,
42 int filter_type, double kaiser_beta){
43 int ph, i;
44 int ph_nb = phase_count % 2 ? phase_count : phase_count / 2 + 1;
45 double x, y, w, t, s;
46 double *tab = av_malloc_array(tap_count+1, sizeof(*tab));
47 double *sin_lut = av_malloc_array(ph_nb, sizeof(*sin_lut));
48 const int center= (tap_count-1)/2;
49 double norm = 0;
50 int ret = AVERROR(ENOMEM);
51
52 if (!tab || !sin_lut)
53 goto fail;
54
55 av_assert0(tap_count == 1 || tap_count % 2 == 0);
56
57 /* if upsampling, only need to interpolate, no filter */
58 if (factor > 1.0)
59 factor = 1.0;
60
61 if (factor == 1.0) {
62 for (ph = 0; ph < ph_nb; ph++)
63 sin_lut[ph] = sin(M_PI * ph / phase_count) * (center & 1 ? 1 : -1);
64 }
65 for(ph = 0; ph < ph_nb; ph++) {
66 s = sin_lut[ph];
67 for(i=0;i<tap_count;i++) {
68 x = M_PI * ((double)(i - center) - (double)ph / phase_count) * factor;
69 if (x == 0) y = 1.0;
70 else if (factor == 1.0)
71 y = s / x;
72 else
73 y = sin(x) / x;
74 switch(filter_type){
76 const float d= -0.5; //first order derivative = -0.5
77 x = fabs(((double)(i - center) - (double)ph / phase_count) * factor);
78 if(x<1.0) y= 1 - 3*x*x + 2*x*x*x + d*( -x*x + x*x*x);
79 else y= d*(-4 + 8*x - 5*x*x + x*x*x);
80 break;}
82 w = 2.0*x / (factor*tap_count);
83 t = -cos(w);
84 y *= 0.3635819 - 0.4891775 * t + 0.1365995 * (2*t*t-1) - 0.0106411 * (4*t*t*t - 3*t);
85 break;
87 w = 2.0*x / (factor*tap_count*M_PI);
88 y *= av_bessel_i0(kaiser_beta*sqrt(FFMAX(1-w*w, 0)));
89 break;
90 default:
91 av_assert0(0);
92 }
93
94 tab[i] = y;
95 s = -s;
96 if (!ph)
97 norm += y;
98 }
99
100 /* normalize so that an uniform color remains the same */
101 switch(c->format){
103 for(i=0;i<tap_count;i++)
104 ((int16_t*)filter)[ph * alloc + i] = av_clip_int16(lrintf(tab[i] * scale / norm));
105 if (phase_count % 2) break;
106 for (i = 0; i < tap_count; i++)
107 ((int16_t*)filter)[(phase_count-ph) * alloc + tap_count-1-i] = ((int16_t*)filter)[ph * alloc + i];
108 break;
110 for(i=0;i<tap_count;i++)
111 ((int32_t*)filter)[ph * alloc + i] = av_clipl_int32(llrint(tab[i] * scale / norm));
112 if (phase_count % 2) break;
113 for (i = 0; i < tap_count; i++)
114 ((int32_t*)filter)[(phase_count-ph) * alloc + tap_count-1-i] = ((int32_t*)filter)[ph * alloc + i];
115 break;
117 for(i=0;i<tap_count;i++)
118 ((float*)filter)[ph * alloc + i] = tab[i] * scale / norm;
119 if (phase_count % 2) break;
120 for (i = 0; i < tap_count; i++)
121 ((float*)filter)[(phase_count-ph) * alloc + tap_count-1-i] = ((float*)filter)[ph * alloc + i];
122 break;
124 for(i=0;i<tap_count;i++)
125 ((double*)filter)[ph * alloc + i] = tab[i] * scale / norm;
126 if (phase_count % 2) break;
127 for (i = 0; i < tap_count; i++)
128 ((double*)filter)[(phase_count-ph) * alloc + tap_count-1-i] = ((double*)filter)[ph * alloc + i];
129 break;
130 }
131 }
132#if 0
133 {
134#define LEN 1024
135 int j,k;
136 double sine[LEN + tap_count];
137 double filtered[LEN];
138 double maxff=-2, minff=2, maxsf=-2, minsf=2;
139 for(i=0; i<LEN; i++){
140 double ss=0, sf=0, ff=0;
141 for(j=0; j<LEN+tap_count; j++)
142 sine[j]= cos(i*j*M_PI/LEN);
143 for(j=0; j<LEN; j++){
144 double sum=0;
145 ph=0;
146 for(k=0; k<tap_count; k++)
147 sum += filter[ph * tap_count + k] * sine[k+j];
148 filtered[j]= sum / (1<<FILTER_SHIFT);
149 ss+= sine[j + center] * sine[j + center];
150 ff+= filtered[j] * filtered[j];
151 sf+= sine[j + center] * filtered[j];
152 }
153 ss= sqrt(2*ss/LEN);
154 ff= sqrt(2*ff/LEN);
155 sf= 2*sf/LEN;
156 maxff= FFMAX(maxff, ff);
157 minff= FFMIN(minff, ff);
158 maxsf= FFMAX(maxsf, sf);
159 minsf= FFMIN(minsf, sf);
160 if(i%11==0){
161 av_log(NULL, AV_LOG_ERROR, "i:%4d ss:%f ff:%13.6e-%13.6e sf:%13.6e-%13.6e\n", i, ss, maxff, minff, maxsf, minsf);
162 minff=minsf= 2;
163 maxff=maxsf= -2;
164 }
165 }
166 }
167#endif
168
169 ret = 0;
170fail:
171 av_free(tab);
172 av_free(sin_lut);
173 return ret;
174}
175
177 ResampleContext *c = *cc;
178 if(!c)
179 return;
180 av_freep(&c->filter_bank);
181 av_freep(cc);
182}
183
184static ResampleContext *resample_init(ResampleContext *c, int out_rate, int in_rate, int filter_size, int phase_shift, int linear,
185 double cutoff0, enum AVSampleFormat format, enum SwrFilterType filter_type, double kaiser_beta,
186 double precision, int cheby, int exact_rational)
187{
188 double cutoff = cutoff0? cutoff0 : 0.97;
189 double factor= FFMIN(out_rate * cutoff / in_rate, 1.0);
190 int phase_count= 1<<phase_shift;
191 int phase_count_compensation = phase_count;
192 int filter_length = FFMAX((int)ceil(filter_size/factor), 1);
193
194 if (filter_length > 1)
195 filter_length = FFALIGN(filter_length, 2);
196
197 if (exact_rational) {
198 int phase_count_exact, phase_count_exact_den;
199
200 av_reduce(&phase_count_exact, &phase_count_exact_den, out_rate, in_rate, INT_MAX);
201 if (phase_count_exact <= phase_count) {
202 phase_count_compensation = phase_count_exact * (phase_count / phase_count_exact);
203 phase_count = phase_count_exact;
204 }
205 }
206
207 if (!c || c->phase_count != phase_count || c->linear!=linear || c->factor != factor
208 || c->filter_length != filter_length || c->format != format
209 || c->filter_type != filter_type || c->kaiser_beta != kaiser_beta) {
211 c = av_mallocz(sizeof(*c));
212 if (!c)
213 return NULL;
214
215 c->format= format;
216
217 c->felem_size= av_get_bytes_per_sample(c->format);
218
219 switch(c->format){
221 c->filter_shift = 15;
222 break;
224 c->filter_shift = 30;
225 break;
228 c->filter_shift = 0;
229 break;
230 default:
231 av_log(NULL, AV_LOG_ERROR, "Unsupported sample format\n");
232 av_assert0(0);
233 }
234
235 if (filter_size/factor > INT32_MAX/256) {
236 av_log(NULL, AV_LOG_ERROR, "Filter length too large\n");
237 goto error;
238 }
239
240 c->phase_count = phase_count;
241 c->linear = linear;
242 c->factor = factor;
243 c->filter_length = filter_length;
244 c->filter_alloc = FFALIGN(c->filter_length, 8);
245 c->filter_bank = av_calloc(c->filter_alloc, (phase_count+1)*c->felem_size);
246 c->filter_type = filter_type;
247 c->kaiser_beta = kaiser_beta;
248 c->phase_count_compensation = phase_count_compensation;
249 if (!c->filter_bank)
250 goto error;
251 if (build_filter(c, (void*)c->filter_bank, factor, c->filter_length, c->filter_alloc, phase_count, 1<<c->filter_shift, filter_type, kaiser_beta))
252 goto error;
253 memcpy(c->filter_bank + (c->filter_alloc*phase_count+1)*c->felem_size, c->filter_bank, (c->filter_alloc-1)*c->felem_size);
254 memcpy(c->filter_bank + (c->filter_alloc*phase_count )*c->felem_size, c->filter_bank + (c->filter_alloc - 1)*c->felem_size, c->felem_size);
255 }
256
257 c->compensation_distance= 0;
258 if(!av_reduce(&c->src_incr, &c->dst_incr, out_rate, in_rate * (int64_t)phase_count, INT32_MAX/2))
259 goto error;
260 while (c->dst_incr < (1<<20) && c->src_incr < (1<<20)) {
261 c->dst_incr *= 2;
262 c->src_incr *= 2;
263 }
264 c->ideal_dst_incr = c->dst_incr;
265 c->dst_incr_div = c->dst_incr / c->src_incr;
266 c->dst_incr_mod = c->dst_incr % c->src_incr;
267
268 c->index= -phase_count*((c->filter_length-1)/2);
269 c->frac= 0;
270
272
273 return c;
274error:
275 av_freep(&c->filter_bank);
276 av_free(c);
277 return NULL;
278}
279
281{
282 uint8_t *new_filter_bank;
283 int new_src_incr, new_dst_incr;
284 int phase_count = c->phase_count_compensation;
285 int ret;
286
287 if (phase_count == c->phase_count)
288 return 0;
289
290 av_assert0(!c->frac && !c->dst_incr_mod);
291
292 new_filter_bank = av_calloc(c->filter_alloc, (phase_count + 1) * c->felem_size);
293 if (!new_filter_bank)
294 return AVERROR(ENOMEM);
295
296 ret = build_filter(c, new_filter_bank, c->factor, c->filter_length, c->filter_alloc,
297 phase_count, 1 << c->filter_shift, c->filter_type, c->kaiser_beta);
298 if (ret < 0) {
299 av_freep(&new_filter_bank);
300 return ret;
301 }
302 memcpy(new_filter_bank + (c->filter_alloc*phase_count+1)*c->felem_size, new_filter_bank, (c->filter_alloc-1)*c->felem_size);
303 memcpy(new_filter_bank + (c->filter_alloc*phase_count )*c->felem_size, new_filter_bank + (c->filter_alloc - 1)*c->felem_size, c->felem_size);
304
305 if (!av_reduce(&new_src_incr, &new_dst_incr, c->src_incr,
306 c->dst_incr * (int64_t)(phase_count/c->phase_count), INT32_MAX/2))
307 {
308 av_freep(&new_filter_bank);
309 return AVERROR(EINVAL);
310 }
311
312 c->src_incr = new_src_incr;
313 c->dst_incr = new_dst_incr;
314 while (c->dst_incr < (1<<20) && c->src_incr < (1<<20)) {
315 c->dst_incr *= 2;
316 c->src_incr *= 2;
317 }
318 c->ideal_dst_incr = c->dst_incr;
319 c->dst_incr_div = c->dst_incr / c->src_incr;
320 c->dst_incr_mod = c->dst_incr % c->src_incr;
321 c->index *= phase_count / c->phase_count;
322 c->phase_count = phase_count;
323 av_freep(&c->filter_bank);
324 c->filter_bank = new_filter_bank;
325 return 0;
326}
327
328static int set_compensation(ResampleContext *c, int sample_delta, int compensation_distance){
329 int ret;
330
331 if (compensation_distance && sample_delta) {
333 if (ret < 0)
334 return ret;
335 }
336
337 c->compensation_distance= compensation_distance;
338 if (compensation_distance)
339 c->dst_incr = c->ideal_dst_incr - c->ideal_dst_incr * (int64_t)sample_delta / compensation_distance;
340 else
341 c->dst_incr = c->ideal_dst_incr;
342
343 c->dst_incr_div = c->dst_incr / c->src_incr;
344 c->dst_incr_mod = c->dst_incr % c->src_incr;
345
346 return 0;
347}
348
349static int multiple_resample(ResampleContext *c, AudioData *dst, int dst_size, AudioData *src, int src_size, int *consumed){
350 int i;
351 int64_t max_src_size = (INT64_MAX/2 / c->phase_count) / c->src_incr;
352
353 if (c->compensation_distance)
354 dst_size = FFMIN(dst_size, c->compensation_distance);
355 src_size = FFMIN(src_size, max_src_size);
356
357 *consumed = 0;
358
359 if (c->filter_length == 1 && c->phase_count == 1) {
360 int64_t index2= (1LL<<32)*c->frac/c->src_incr + (1LL<<32)*c->index + 1;
361 int64_t incr= (1LL<<32) * c->dst_incr / c->src_incr + 1;
362 int new_size = (src_size * (int64_t)c->src_incr - c->frac + c->dst_incr - 1) / c->dst_incr;
363
364 dst_size = FFMAX(FFMIN(dst_size, new_size), 0);
365 if (dst_size > 0) {
366 for (i = 0; i < dst->ch_count; i++) {
367 c->dsp.resample_one(dst->ch[i], src->ch[i], dst_size, index2, incr);
368 if (i+1 == dst->ch_count) {
369 c->index += dst_size * c->dst_incr_div;
370 c->index += (c->frac + dst_size * (int64_t)c->dst_incr_mod) / c->src_incr;
371 av_assert2(c->index >= 0);
372 *consumed = c->index;
373 c->frac = (c->frac + dst_size * (int64_t)c->dst_incr_mod) % c->src_incr;
374 c->index = 0;
375 }
376 }
377 }
378 } else {
379 int64_t end_index = (1LL + src_size - c->filter_length) * c->phase_count;
380 int64_t delta_frac = (end_index - c->index) * c->src_incr - c->frac;
381 int delta_n = (delta_frac + c->dst_incr - 1) / c->dst_incr;
382 int (*resample_func)(struct ResampleContext *c, void *dst,
383 const void *src, int n, int update_ctx);
384
385 dst_size = FFMAX(FFMIN(dst_size, delta_n), 0);
386 if (dst_size > 0) {
387 /* resample_linear and resample_common should have same behavior
388 * when frac and dst_incr_mod are zero */
389 resample_func = (c->linear && (c->frac || c->dst_incr_mod)) ?
390 c->dsp.resample_linear : c->dsp.resample_common;
391 for (i = 0; i < dst->ch_count; i++)
392 *consumed = resample_func(c, dst->ch[i], src->ch[i], dst_size, i+1 == dst->ch_count);
393 }
394 }
395
396 if (c->compensation_distance) {
397 c->compensation_distance -= dst_size;
398 if (!c->compensation_distance) {
399 c->dst_incr = c->ideal_dst_incr;
400 c->dst_incr_div = c->dst_incr / c->src_incr;
401 c->dst_incr_mod = c->dst_incr % c->src_incr;
402 }
403 }
404
405 return dst_size;
406}
407
409 ResampleContext *c = s->resample;
410 int64_t num = s->in_buffer_count - (c->filter_length-1)/2;
411 num *= c->phase_count;
412 num -= c->index;
413 num *= c->src_incr;
414 num -= c->frac;
415 return av_rescale(num, base, s->in_sample_rate*(int64_t)c->src_incr * c->phase_count);
416}
417
418static int64_t get_out_samples(struct SwrContext *s, int in_samples) {
419 ResampleContext *c = s->resample;
420 // The + 2 are added to allow implementations to be slightly inaccurate, they should not be needed currently.
421 // They also make it easier to proof that changes and optimizations do not
422 // break the upper bound.
423 int64_t num = s->in_buffer_count + 2LL + in_samples;
424 num *= c->phase_count;
425 num -= c->index;
426 num = av_rescale_rnd(num, s->out_sample_rate, ((int64_t)s->in_sample_rate) * c->phase_count, AV_ROUND_UP) + 2;
427
428 if (c->compensation_distance) {
429 if (num > INT_MAX)
430 return AVERROR(EINVAL);
431
432 num = FFMAX(num, (num * c->ideal_dst_incr - 1) / c->dst_incr + 1);
433 }
434 return num;
435}
436
437static int resample_flush(struct SwrContext *s) {
438 ResampleContext *c = s->resample;
439 AudioData *a= &s->in_buffer;
440 int i, j, ret;
441 int reflection = (FFMIN(s->in_buffer_count, c->filter_length) + 1) / 2;
442
443 if((ret = swri_realloc_audio(a, s->in_buffer_index + s->in_buffer_count + reflection)) < 0)
444 return ret;
445 av_assert0(a->planar);
446 for(i=0; i<a->ch_count; i++){
447 for(j=0; j<reflection; j++){
448 memcpy(a->ch[i] + (s->in_buffer_index+s->in_buffer_count+j )*a->bps,
449 a->ch[i] + (s->in_buffer_index+s->in_buffer_count-j-1)*a->bps, a->bps);
450 }
451 }
452 s->in_buffer_count += reflection;
453 return 0;
454}
455
456// in fact the whole handle multiple ridiculously small buffers might need more thinking...
458 int in_count, int *out_idx, int *out_sz)
459{
460 int n, ch, num = FFMIN(in_count + *out_sz, c->filter_length + 1), res;
461
462 if (c->index >= 0)
463 return 0;
464
465 if ((res = swri_realloc_audio(dst, c->filter_length * 2 + 1)) < 0)
466 return res;
467
468 // copy
469 for (n = *out_sz; n < num; n++) {
470 for (ch = 0; ch < src->ch_count; ch++) {
471 memcpy(dst->ch[ch] + ((c->filter_length + n) * c->felem_size),
472 src->ch[ch] + ((n - *out_sz) * c->felem_size), c->felem_size);
473 }
474 }
475
476 // if not enough data is in, return and wait for more
477 if (num < c->filter_length + 1) {
478 *out_sz = num;
479 *out_idx = c->filter_length;
480 return INT_MAX;
481 }
482
483 // else invert
484 for (n = 1; n <= c->filter_length; n++) {
485 for (ch = 0; ch < src->ch_count; ch++) {
486 memcpy(dst->ch[ch] + ((c->filter_length - n) * c->felem_size),
487 dst->ch[ch] + ((c->filter_length + n) * c->felem_size),
488 c->felem_size);
489 }
490 }
491
492 res = num - *out_sz;
493 *out_idx = c->filter_length;
494 while (c->index < 0) {
495 --*out_idx;
496 c->index += c->phase_count;
497 }
498 *out_sz = FFMAX(*out_sz + c->filter_length,
499 1 + c->filter_length * 2) - *out_idx;
500
501 return FFMAX(res, 0);
502}
503
504const struct Resampler swri_resampler = {
505 .init = resample_init,
506 .free = resample_free,
507 .multiple_resample = multiple_resample,
508 .flush = resample_flush,
509 .set_compensation = set_compensation,
510 .get_delay = get_delay,
511 .invert_initial_buffer = invert_initial_buffer,
512 .get_out_samples = get_out_samples,
513};
uint8_t ptrdiff_t const uint8_t ptrdiff_t int intptr_t intptr_t int int16_t * dst
Definition dsp.h:87
static const char *const format[]
Definition af_aiir.c:444
#define LEN
static float kaiser_beta(float att, float tr_bw)
Definition asrc_sinc.c:125
int32_t
simple assert() macros that are a bit more flexible than ISO C assert().
#define av_assert2(cond)
assert() equivalent, that does lie in speed critical code.
Definition avassert.h:68
#define av_assert0(cond)
assert() equivalent, that is always enabled.
Definition avassert.h:42
#define i(width, name, range_min, range_max)
Definition cbs_h264.c:63
static int FUNC ph(CodedBitstreamContext *ctx, RWContext *rw, H266RawPH *current)
#define ss(width, name, subs,...)
Definition cbs_vp9.c:202
#define s(width, name)
Definition cbs_vp9.c:198
#define av_clipl_int32
Definition common.h:118
#define av_clip_int16
Definition common.h:115
#define NULL
Definition coverity.c:32
long long int64_t
Definition coverity.c:34
static __device__ float ceil(float a)
static __device__ float fabs(float a)
#define fail
Definition test.h:479
#define AVERROR(e)
Definition error.h:45
#define AV_LOG_ERROR
Something went wrong and cannot losslessly be recovered.
Definition log.h:210
int av_reduce(int *dst_num, int *dst_den, int64_t num, int64_t den, int64_t max)
Reduce a fraction.
Definition rational.c:35
int64_t av_rescale(int64_t a, int64_t b, int64_t c)
Rescale a 64-bit integer with rounding to nearest.
int64_t av_rescale_rnd(int64_t a, int64_t b, int64_t c, enum AVRounding rnd)
Rescale a 64-bit integer with specified rounding.
Definition mathematics.c:58
double av_bessel_i0(double x)
0th order modified bessel function of the first kind.
@ AV_ROUND_UP
Round toward +infinity.
int av_get_bytes_per_sample(enum AVSampleFormat sample_fmt)
Return number of bytes per sample.
Definition samplefmt.c:108
AVSampleFormat
Audio sample formats.
Definition samplefmt.h:55
@ AV_SAMPLE_FMT_FLTP
float, planar
Definition samplefmt.h:66
@ AV_SAMPLE_FMT_S16P
signed 16 bits, planar
Definition samplefmt.h:64
@ AV_SAMPLE_FMT_S32P
signed 32 bits, planar
Definition samplefmt.h:65
@ AV_SAMPLE_FMT_DBLP
double, planar
Definition samplefmt.h:67
SwrFilterType
Resampling Filter Types.
Definition swresample.h:173
@ SWR_FILTER_TYPE_BLACKMAN_NUTTALL
Blackman Nuttall windowed sinc.
Definition swresample.h:175
@ SWR_FILTER_TYPE_CUBIC
Cubic.
Definition swresample.h:174
@ SWR_FILTER_TYPE_KAISER
Kaiser windowed sinc.
Definition swresample.h:176
int a
static int linear(InterplayACMContext *s, unsigned ind, unsigned col)
static void scale(int *out, const int *in, const int w, const int h, const int shift)
Definition intra.c:278
static const int factor[16]
Definition vf_pp7.c:98
#define llrint(x)
Definition libm.h:396
#define lrintf(x)
Definition libm_mips.h:72
uint8_t w
Definition llvidencdsp.c:39
#define FFMIN(a, b)
Definition macros.h:49
#define FFMAX(a, b)
Definition macros.h:47
#define FFALIGN(x, a)
Definition macros.h:78
#define M_PI
Definition mathematics.h:67
void * av_calloc(size_t nmemb, size_t size)
Definition mem.c:264
Memory handling functions.
static int64_t get_delay(struct SwrContext *s, int64_t base)
Definition resample.c:408
static int multiple_resample(ResampleContext *c, AudioData *dst, int dst_size, AudioData *src, int src_size, int *consumed)
Definition resample.c:349
static int invert_initial_buffer(ResampleContext *c, AudioData *dst, const AudioData *src, int in_count, int *out_idx, int *out_sz)
Definition resample.c:457
static int rebuild_filter_bank_with_compensation(ResampleContext *c)
Definition resample.c:280
static int set_compensation(ResampleContext *c, int sample_delta, int compensation_distance)
Definition resample.c:328
const struct Resampler swri_resampler
Definition resample.c:504
static int resample_flush(struct SwrContext *s)
Definition resample.c:437
static int64_t get_out_samples(struct SwrContext *s, int in_samples)
Definition resample.c:418
static ResampleContext * resample_init(ResampleContext *c, int out_rate, int in_rate, int filter_size, int phase_shift, int linear, double cutoff0, enum AVSampleFormat format, enum SwrFilterType filter_type, double kaiser_beta, double precision, int cheby, int exact_rational)
Definition resample.c:184
static void resample_free(ResampleContext **cc)
Definition resample.c:176
static int build_filter(ResampleContext *c, void *filter, double factor, int tap_count, int alloc, int phase_count, int scale, int filter_type, double kaiser_beta)
builds a polyphase filterbank.
Definition resample.c:41
void swri_resample_dsp_init(ResampleContext *c)
The libswresample context.
int swri_realloc_audio(AudioData *a, int count)
Definition swresample.c:408
#define av_free(p)
#define av_malloc_array(a, b)
#define av_mallocz(s)
#define av_freep(p)
#define av_log(a,...)
static void error(const char *err)
void(* filter)(uint8_t *src, ptrdiff_t stride, int qscale)
Definition h263dsp.c:29
#define src
Definition vp8dsp.c:248
static const struct twinvq_data tab
uint8_t base
Definition vp3data.h:128
static double c[64]